@@ -1,14 +1,31 @@
 #!./kitty/launcher/kitty +launch
 # License: GPLv3 Copyright: 2022, Kovid Goyal <kovid at kovidgoyal.net>
 
+import bz2
 import io
 import json
 import os
+import re
+import struct
 import subprocess
 import sys
+import tarfile
 from contextlib import contextmanager, suppress
 from functools import lru_cache
-from typing import Any, Dict, Iterator, List, Optional, Sequence, Set, Tuple, Union
+from itertools import chain
+from typing import (
+    Any,
+    BinaryIO,
+    Dict,
+    Iterator,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    TextIO,
+    Tuple,
+    Union,
+)
 
 import kitty.constants as kc
 from kittens.tui.operations import Mode
@@ -20,7 +37,9 @@ from kitty.cli import (
     parse_option_spec,
     serialize_as_go_string,
 )
-from kitty.guess_mime_type import text_mimes
+from kitty.conf.generate import gen_go_code
+from kitty.conf.types import Definition
+from kitty.guess_mime_type import known_extensions, text_mimes
 from kitty.key_encoding import config_mod_map
 from kitty.key_names import character_key_name_aliases, functional_key_name_aliases
 from kitty.options.types import Options
@@ -31,6 +50,19 @@ from kitty.rgb import color_names
 changed: List[str] = []
 
 
+def newer(dest: str, *sources: str) -> bool:
+    try:
+        dtime = os.path.getmtime(dest)
+    except OSError:
+        return True
+    for s in chain(sources, (__file__,)):
+        with suppress(FileNotFoundError):
+            if os.path.getmtime(s) >= dtime:
+                return True
+    return False
+
+
+
 # Utils {{{
 
 def serialize_go_dict(x: Union[Dict[str, int], Dict[int, str], Dict[int, int], Dict[str, str]]) -> str:
@@ -303,9 +335,46 @@ def wrapped_kittens() -> Sequence[str]:
     raise Exception('Failed to read wrapped kittens from kitty wrapper script')
 
 
+def generate_conf_parser(kitten: str, defn: Definition) -> None:
+    with replace_if_needed(f'kittens/{kitten}/conf_generated.go'):
+        print(f'package {kitten}')
+        print(gen_go_code(defn))
+
+
+def generate_extra_cli_parser(name: str, spec: str) -> None:
+    print('import "kitty/tools/cli"')
+    go_opts = tuple(go_options_for_seq(parse_option_spec(spec)[0]))
+    print(f'type {name}_options struct ''{')
+    for opt in go_opts:
+        print(opt.struct_declaration())
+    print('}')
+    print(f'func parse_{name}_args(args []string) (*{name}_options, []string, error) ''{')
+    print(f'root := cli.Command{{Name: `{name}` }}')
+    for opt in go_opts:
+        print(opt.as_option('root'))
+    print('cmd, err := root.ParseArgs(args)')
+    print('if err != nil { return nil, nil, err }')
+    print(f'var opts {name}_options')
+    print('err = cmd.GetOptionValues(&opts)')
+    print('if err != nil { return nil, nil, err }')
+    print('return &opts, cmd.Args, nil')
+    print('}')
+
+
 def kitten_clis() -> None:
+    from kittens.runner import get_kitten_conf_docs, get_kitten_extra_cli_parsers
     for kitten in wrapped_kittens():
-        with replace_if_needed(f'tools/cmd/{kitten}/cli_generated.go'):
+        defn = get_kitten_conf_docs(kitten)
+        if defn is not None:
+            generate_conf_parser(kitten, defn)
+        ecp = get_kitten_extra_cli_parsers(kitten)
+        if ecp:
+            for name, spec in ecp.items():
+                with replace_if_needed(f'kittens/{kitten}/{name}_cli_generated.go'):
+                    print(f'package {kitten}')
+                    generate_extra_cli_parser(name, spec)
+
+        with replace_if_needed(f'kittens/{kitten}/cli_generated.go'):
             od = []
             kcd = kitten_cli_docs(kitten)
             has_underscore = '_' in kitten
@@ -314,10 +383,11 @@ def kitten_clis() -> None:
             print('func create_cmd(root *cli.Command, run_func func(*cli.Command, *Options, []string)(int, error)) {')
             print('ans := root.AddSubCommand(&cli.Command{')
             print(f'Name: "{kitten}",')
-            print(f'ShortDescription: "{serialize_as_go_string(kcd["short_desc"])}",')
-            if kcd['usage']:
-                print(f'Usage: "[options] {serialize_as_go_string(kcd["usage"])}",')
-            print(f'HelpText: "{serialize_as_go_string(kcd["help_text"])}",')
+            if kcd:
+                print(f'ShortDescription: "{serialize_as_go_string(kcd["short_desc"])}",')
+                if kcd['usage']:
+                    print(f'Usage: "[options] {serialize_as_go_string(kcd["usage"])}",')
+                print(f'HelpText: "{serialize_as_go_string(kcd["help_text"])}",')
             print('Run: func(cmd *cli.Command, args []string) (int, error) {')
             print('opts := Options{}')
             print('err := cmd.GetOptionValues(&opts)')
@@ -336,6 +406,8 @@ def kitten_clis() -> None:
                 print("clone := root.AddClone(ans.Group, ans)")
                 print('clone.Hidden = false')
                 print(f'clone.Name = "{serialize_as_go_string(kitten.replace("_", "-"))}"')
+            if not kcd:
+                print('specialize_command(ans)')
             print('}')
             print('type Options struct {')
             print('\n'.join(od))
@@ -368,11 +440,24 @@ def generate_spinners() -> str:
 
 
 def generate_color_names() -> str:
+    selfg = "" if Options.selection_foreground is None else Options.selection_foreground.as_sharp
+    selbg = "" if Options.selection_background is None else Options.selection_background.as_sharp
+    cursor = "" if Options.cursor is None else Options.cursor.as_sharp
     return 'package style\n\nvar ColorNames = map[string]RGBA{' + '\n'.join(
         f'\t"{name}": RGBA{{ Red:{val.red}, Green:{val.green}, Blue:{val.blue} }},'
         for name, val in color_names.items()
     ) + '\n}' + '\n\nvar ColorTable = [256]uint32{' + ', '.join(
-        f'{x}' for x in Options.color_table) + '}\n'
+        f'{x}' for x in Options.color_table) + '}\n' + f'''
+var DefaultColors = struct {{
+Foreground, Background, Cursor, SelectionFg, SelectionBg string
+}}{{
+Foreground: "{Options.foreground.as_sharp}",
+Background: "{Options.background.as_sharp}",
+Cursor: "{cursor}",
+SelectionFg: "{selfg}",
+SelectionBg: "{selbg}",
+}}
+'''
 
 
 def load_ref_map() -> Dict[str, Dict[str, str]]:
@@ -384,8 +469,17 @@ def load_ref_map() -> Dict[str, Dict[str, str]]:
 
 
 def generate_constants() -> str:
+    from kittens.hints.main import DEFAULT_REGEX
+    from kitty.options.types import Options
+    from kitty.options.utils import allowed_shell_integration_values
+    del sys.modules['kittens.hints.main']
     ref_map = load_ref_map()
+    with open('kitty/data-types.h') as dt:
+        m = re.search(r'^#define IMAGE_PLACEHOLDER_CHAR (\S+)', dt.read(), flags=re.M)
+        assert m is not None
+        placeholder_char = int(m.group(1), 16)
     dp = ", ".join(map(lambda x: f'"{serialize_as_go_string(x)}"', kc.default_pager_for_help))
+    url_prefixes = ','.join(f'"{x}"' for x in Options.url_prefixes)
     return f'''\
 package kitty
 
@@ -394,11 +488,14 @@ type VersionType struct {{
 }}
 const VersionString string = "{kc.str_version}"
 const WebsiteBaseURL string = "{kc.website_base_url}"
+const ImagePlaceholderChar rune = {placeholder_char}
 const VCSRevision string = ""
+const SSHControlMasterTemplate = "{kc.ssh_control_master_template}"
 const RC_ENCRYPTION_PROTOCOL_VERSION string = "{kc.RC_ENCRYPTION_PROTOCOL_VERSION}"
 const IsFrozenBuild bool = false
 const IsStandaloneBuild bool = false
 const HandleTermiosSignals = {Mode.HANDLE_TERMIOS_SIGNALS.value[0]}
+const HintsDefaultRegex = `{DEFAULT_REGEX}`
 var Version VersionType = VersionType{{Major: {kc.version.major}, Minor: {kc.version.minor}, Patch: {kc.version.patch},}}
 var DefaultPager []string = []string{{ {dp} }}
 var FunctionalKeyNameAliases = map[string]string{serialize_go_dict(functional_key_name_aliases)}
@@ -406,6 +503,15 @@ var CharacterKeyNameAliases = map[string]string{serialize_go_dict(character_key_
 var ConfigModMap = map[string]uint16{serialize_go_dict(config_mod_map)}
 var RefMap = map[string]string{serialize_go_dict(ref_map['ref'])}
 var DocTitleMap = map[string]string{serialize_go_dict(ref_map['doc'])}
+var AllowedShellIntegrationValues = []string{{ {str(sorted(allowed_shell_integration_values))[1:-1].replace("'", '"')} }}
+var KittyConfigDefaults = struct {{
+Term, Shell_integration, Select_by_word_characters string
+Wheel_scroll_multiplier int
+Url_prefixes []string
+}}{{
+Term: "{Options.term}", Shell_integration: "{' '.join(Options.shell_integration)}", Url_prefixes: []string{{ {url_prefixes} }},
+Select_by_word_characters: `{Options.select_by_word_characters}`, Wheel_scroll_multiplier: {Options.wheel_scroll_multiplier},
+}}
 '''  # }}}
 
 
@@ -580,9 +686,59 @@ def generate_textual_mimetypes() -> str:
     for k in text_mimes:
         ans.append(f'  "{serialize_as_go_string(k)}": true,')
     ans.append('}')
+    ans.append('var KnownExtensions = map[string]string{')
+    for k, v in known_extensions.items():
+        ans.append(f'  ".{serialize_as_go_string(k)}": "{serialize_as_go_string(v)}",')
+    ans.append('}')
     return '\n'.join(ans)
 
 
+def write_compressed_data(data: bytes, d: BinaryIO) -> None:
+    d.write(struct.pack('<I', len(data)))
+    d.write(bz2.compress(data))
+
+
+def generate_unicode_names(src: TextIO, dest: BinaryIO) -> None:
+    num_names, num_of_words = map(int, next(src).split())
+    gob = io.BytesIO()
+    gob.write(struct.pack('<II', num_names, num_of_words))
+    for line in src:
+        line = line.strip()
+        if line:
+            a, aliases = line.partition('\t')[::2]
+            cp, name = a.partition(' ')[::2]
+            ename = name.encode()
+            record = struct.pack('<IH', int(cp), len(ename)) + ename
+            if aliases:
+                record += aliases.encode()
+            gob.write(struct.pack('<H', len(record)) + record)
+    write_compressed_data(gob.getvalue(), dest)
+
+
+def generate_ssh_kitten_data() -> None:
+    files = {
+        'terminfo/kitty.terminfo', 'terminfo/x/xterm-kitty',
+    }
+    for dirpath, dirnames, filenames in os.walk('shell-integration'):
+        for f in filenames:
+            path = os.path.join(dirpath, f)
+            files.add(path.replace(os.sep, '/'))
+    dest = 'kittens/ssh/data_generated.bin'
+
+    def normalize(t: tarfile.TarInfo) -> tarfile.TarInfo:
+        t.uid = t.gid = 0
+        t.uname = t.gname = ''
+        return t
+
+    if newer(dest, *files):
+        buf = io.BytesIO()
+        with tarfile.open(fileobj=buf, mode='w') as tf:
+            for f in sorted(files):
+                tf.add(f, filter=normalize)
+        with open(dest, 'wb') as d:
+            write_compressed_data(buf.getvalue(), d)
+
+
 def main() -> None:
     with replace_if_needed('constants_generated.go') as f:
         f.write(generate_constants())
@@ -596,6 +752,10 @@ def main() -> None:
         f.write(generate_mimetypes())
     with replace_if_needed('tools/utils/mimetypes_textual_generated.go') as f:
         f.write(generate_textual_mimetypes())
+    if newer('tools/unicode_names/data_generated.bin', 'tools/unicode_names/names.txt'):
+        with open('tools/unicode_names/data_generated.bin', 'wb') as dest, open('tools/unicode_names/names.txt') as src:
+            generate_unicode_names(src, dest)
+    generate_ssh_kitten_data()
 
     update_completion()
     update_at_commands()
